#!/usr/bin/perl -w

# $Id: beadstudio2ibs.pl 944 2009-01-28 12:19:31Z martin $

# Copyright (C) 2009 Markus Ringnér

# This file is part of Illumina plug-in package for BASE.
# Available at http://baseplugins.thep.lu.se/
# BASE main site: http://base.thep.lu.se/

# This is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# The software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this package. If not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use File::Spec;
use Getopt::Long;
use IO::File;
use Pod::Usage;

# initialize user-definable parameters to default values

my $samples_file='sample_probe_profile.txt';
my $controls_file='control_probe_profile.txt';

# set user-defined parameters

GetOptions('samples_file=s' => \$samples_file,
	   'controls_file=s' => \$controls_file) 
    or pod2usage(1);

$samples_file=File::Spec->canonpath($samples_file);
$controls_file=File::Spec->canonpath($controls_file);

if($samples_file ne '') {
    unless((-r $samples_file)) {
	print STDERR "Error: Cannot read samples data file: $samples_file\n";
	exit(0);
    }
}
else {
    print STDERR "Error: No file with data for samples specified!\n";
    pod2usage(1);
}

if($controls_file ne '') {
    unless((-r $controls_file)) {
	print STDERR "Error: Cannot read controls data file: $controls_file\n";
	exit(0);
    }
}
else {
    print STDERR "Error: No file with data for controls specified!\n";
    pod2usage(1);
}


# Main program - Generate an IBS for each sample

# Read header for samples file

open(SAMPLES,"$samples_file");
my $header=<SAMPLES>;
chomp($header);
$header=~s/[\r\n]+$//;
my $samples_file_error="The samples data file $samples_file is not in the " .
    "correct format";
my @header=split(/\t/,$header,-1);
if($header[0] ne 'PROBE_ID') {
    print STDERR "$samples_file_error\n";
    print STDERR "The first column header should be " .
	"PROBE_ID, but is $header[0].\n";
    exit(0);
}
if($header[scalar(@header)-1] ne 'ProbeID') {
    print STDERR "$samples_file_error\n";
    print STDERR "The last column header should be " .
	"ProbeID, but is $header[scalar(@header)-1].\n";
    exit(0);
}


my @samples;
for(my $i=1;$i<scalar(@header)-1;$i+=3) {    
    my $header_error=0;
    if($header[$i] !~ /\.AVG_Signal/) {
	$header_error=1;
    }
    my $sample_name=$header[$i];
    $sample_name=~s/\.AVG_Signal//;
    if($header[$i+1]!~/$sample_name\.BEAD_STDERR/  || 
       $header[$i+2]!~/$sample_name\.Avg_NBEADS/) {
	$header_error=1;
    } 
    if($header_error) {
	print STDERR "$samples_file_error\n";
	print STDERR "Columns $i-" . ($i+2) . 
	    " header, $header[$i]\t$header[$i+1]\t$header[$i+2]," .
	    " should be " .
	    "${sample_name}.AVG_signal\t${sample_name}.BEAD_STDERR" .
	    "\t${sample_name}.AVG_NBEADS\n";
	exit(0);
    }
    push(@samples,$sample_name);
}

# Read and write data for sample probes.

my %sample_probes;# Keep track of which probes have been printed
my @file_handles;
foreach my $sample (@samples) {
    my $filename=File::Spec->canonpath("${sample}.csv");
    my $fh=IO::File->new(">$filename");
    if(!defined($fh)) {
        print STDERR "Error: Cannot open file $filename for writing\n";
        exit(0);
    }
    print $fh "Illumicode,N,Mean GRN,Dev GRN\n";
    push(@file_handles,$fh);
}

while(my $line=<SAMPLES>) {
    chomp($line);
    $line=~s/[\r\n]+$//;
    my @line=split(/\t/,$line,-1);
    if(scalar(@header) != scalar(@line)) {
        print STDERR "$samples_file_error\n";
        print STDERR "Not the same number of columns in all rows in $samples_file\n";
        exit(0);
    }
    for(my $i=1;$i<scalar(@line)-1;$i+=3) {    
        my $fh=$file_handles[($i-1)/3];
	my $dev=$line[$i+1]*sqrt($line[$i+2]);
	$sample_probes{$line[scalar(@line)-1]}=1;
        print $fh "$line[scalar(@line)-1],$line[$i+2]," . 
	    $line[$i] . "," . $dev. "\n";
    }
}

close(SAMPLES);

# Read header for control probe data.

open(CONTROLS,"$controls_file");
$header=<CONTROLS>;
chomp($header);
$header=~s/[\r\n]+$//;
my $controls_file_error="The controls data file $controls_file is not in the " .
    "correct format";
@header=split(/\t/,$header,-1);
if($header[0] ne 'ProbeID') {
    print STDERR "$controls_file_error\n";
    print STDERR "The first column header should be " .
	"ProbeID, but is $header[0].\n";
    exit(0);
}

my $i=1;
foreach my $sample (@samples) {
    my $header_error=0;
    if($header[$i]!~/$sample\.AVG_Signal/ ||
       $header[$i+1]!~/$sample\.BEAD_STDERR/  || 
       $header[$i+2]!~/$sample\.Avg_NBEADS/) {
	$header_error=1;
    } 
    if($header_error) {
	print STDERR "$controls_file_error\n";
	print STDERR "Columns $i-" . ($i+2) . 
	    " header, $header[$i]\t$header[$i+1]\t$header[$i+2]," .
	    " should be " .
	    "${sample}.AVG_signal\t${sample}.BEAD_STDERR" .
	    "\t${sample}.AVG_NBEADS\n";
	exit(0);
    }
    $i+=3;
}


# Read and write data for control probes.  

# Some probes are both annotated as sample_probes and control_probes;
# data for these should only appear once in the IBS-file. Some
# controls are annotated as multiple types of controls; data for these
# should only appear once in the IBS-file.

while(my $line=<CONTROLS>) {
    chomp($line);
    $line=~s/[\r\n]+$//;
    my @line=split(/\t/,$line,-1);
    if(scalar(@header) != scalar(@line)) {
        print STDERR "$controls_file_error\n";
        print STDERR "Not the same number of columns in all rows in $controls_file\n";
        exit(0);
    }
    for(my $i=1;$i<scalar(@line);$i+=3) {    
        my $fh=$file_handles[($i-1)/3];
	my $dev=$line[$i+1]*sqrt($line[$i+2]);
	if(!exists($sample_probes{$line[0]})) {
	    $sample_probes{$line[0]}=1;
	    print $fh "$line[0],$line[$i+2]," . $line[$i] . 
		"," . $dev . "\n";
	}
    }
}

close(SAMPLES);


foreach my $fh (@file_handles) {
    close($fh);
}

# Documentation

=pod

=head1 NAME

beadstudio2ibs.pl - split multiple samples BeadStudio data files into separate IBS files for each sample

=head1 SYNOPSIS

beadstudio2ibs.pl --samples_file=<name>  --controls_file=<name> 

=head1 OPTIONS

=over 8

=item B<--samples_file=<name>>

Specify a file with sample probe data from BeadStudio.

=item B<--controls_file=<name>>

Specify a file with controls probe data from BeadStudio.

=back

=head1 AUTHORS 

Markus Ringner

Please report bugs to markus.ringner@med.lu.se

=cut

# End of documentation
